from .models import Course, CourseRegistration, Semester, Session, Invoice, Programme, Spreadsheet, Level, Department
from django.shortcuts import render, redirect, get_object_or_404
from users.models import Person
from result.models import Score
from .forms import CourseFormUnderGrad, UploadForm, SpreadsheetForm, CourseFormHoD
from .filters import ProgrammeFilterVC, ProgrammeFilterDean, CourseFilterDean, SemesterFilter, CourseAcademicFilter, CourseUnderGradFilter, CourseRegisteredUnderGradFilter, InvoiceAdminFilter, ProgrammeFilter, CourseFilterHoD
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
# from django.core.mail import send_mail
# from django.template.loader import render_to_string
from django.core.paginator import Paginator
import csv
from django.http import HttpResponseRedirect, HttpResponse
import os
from pathlib import Path
BASE_DIR = Path(__file__).resolve().parent.parent
from django.urls import reverse
from decimal import Decimal
from django.db.models import Sum, Count, Q

@login_required
def showCoursesAcademic(request):
    if request.user.type == "Academic Staff":
        context = {}
        filtered_courses = CourseAcademicFilter(
            request.GET,
            queryset = Course.objects.filter(lecturer=request.user)
        )
        context['filtered_courses'] = filtered_courses
        paginated_filtered_courses = Paginator(filtered_courses.qs, 200)
        page_number = request.GET.get('page')
        courses_page_obj = paginated_filtered_courses.get_page(page_number)
        context['courses_page_obj'] = courses_page_obj
        total_courses = filtered_courses.qs.count()
        context['total_courses'] = total_courses

        return render(request, 'management/academic_courses.html', context=context)
    else:
        return redirect('index')

@login_required
def showCoursesAdmin(request):
    if request.user.type == "A--":
        context = {}
        filtered_courses = CourseAcademicFilter(
            request.GET,
            queryset = Course.objects.all()
        )
        context['filtered_courses'] = filtered_courses
        paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
        page_number = request.GET.get('page')
        courses_page_obj = paginated_filtered_courses.get_page(page_number)
        context['courses_page_obj'] = courses_page_obj
        total_courses = filtered_courses.qs.count()
        context['total_courses'] = total_courses

        return render(request, 'management/admin_courses.html', context=context)
    else:
        return redirect('index')

@login_required
def showCoursesUnderGrad(request):
    context = {}
    filtered_courses = CourseUnderGradFilter(
        request.GET,
        queryset = Course.objects.filter(open=True)
    )
    context['filtered_courses'] = filtered_courses
    paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
    page_number = request.GET.get('page')
    courses_page_obj = paginated_filtered_courses.get_page(page_number)
    context['courses_page_obj'] = courses_page_obj
    total_courses = filtered_courses.qs.count()
    context['total_courses'] = total_courses
    return render(request, 'management/undergrad_courses.html', context=context)

@login_required
def registerCourse(request, id):
    level = request.user.level
    h100 = request.user.h100
    r100 = request.user.r100
    h200 = request.user.h200
    r200 = request.user.r200
    h300 = request.user.h300
    r300 = request.user.r300
    h400 = request.user.h400
    r400 = request.user.r400

    if level.level == 100:
        if h100 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')
    elif level.level == 200:
        if h100 == True and r100 == True and h200 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')
    elif level.level == 300:
        if h100 == True and r100 == True and h200 == True and r200 == True and h300 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')
    elif level.level == 400:
        if h100 == True and r100 == True and h200 == True and r200 == True and h300 == True and r300 == True and h400 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')

@login_required
def registerCourseRain(request, id):
    level = request.user.level
    h100 = request.user.h100
    r100 = request.user.r100
    h200 = request.user.h200
    r200 = request.user.r200
    h300 = request.user.h300
    r300 = request.user.r300
    h400 = request.user.h400
    r400 = request.user.r400

    if level.level == 100:
        if h100 == True and r100 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')
    elif level.level == 200:
        if h100 == True and r100 == True and h200 == True and r200 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')
    elif level.level == 300:
        if h100 == True and r100 == True and h200 == True and r200 == True and h300 == True and r300 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')
    elif level.level == 400:
        if h100 == True and r100 == True and h200 == True and r200 == True and h300 == True and r300 == True and h400 == True and r400 == True:
            course = Course.objects.get(id=id)
            course_code = course.course_code
            semester = Semester.objects.get(current=True)
            session = Session.objects.get(current=True)

            form = CourseFormUnderGrad()
            if request.method == 'POST':
                form = CourseFormUnderGrad(request.POST or None)
                if form.is_valid():
                    form.save(commit=False).course = course
                    form.save(commit=False).session = session
                    form.save(commit=False).semester = semester
                    form.save(commit=False).undergrad = request.user
                    form.save(commit=False).static_level = request.user.level
                    form.save()

                    reg = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[0]
                    try:
                        reg2 = CourseRegistration.objects.filter(session=session, semester=semester, undergrad=request.user, course=course)[1]
                        messages.error(request, "You have already registered for " + str(course_code))
                        reg.delete()
                    except:
                        pass

                    # email = reg.user.email
                    # name = reg.user.first_name
                    # order_Id = reg.order_Id
                    # send_mail(s
                    #     'Order Status',
                    #     'Dear ' + str(name) + ', Your order status has now been changed to ' + str(status),
                    #     'admin@buildqwik.ng',
                    #     [email, 'support@buildqwik.ng'],
                    #     fail_silently=False,
                    #     html_message = render_to_string('orders/add_order_email.html', {'name': str(name), 'status': str(status), 'order_Id': str(order_Id)})
                    # )
                        messages.success(request, "The course has been registered successfully")
                    return redirect('courses_undergrads')
            return render(request, 'management/course_confirmation.html', {'form': form, 'course': course, 'session': session, 'semester': semester})
        else:
            return redirect('plans')

@login_required
def showCoursesRegisteredUnderGrad100Harm(request):
    context = {}
    filtered_courses = CourseRegisteredUnderGradFilter(
        request.GET,
        queryset = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=100)
    )
    context['filtered_courses'] = filtered_courses
    paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
    page_number = request.GET.get('page')
    courses_page_obj = paginated_filtered_courses.get_page(page_number)
    context['courses_page_obj'] = courses_page_obj
    total_courses = filtered_courses.qs.count()
    context['total_courses'] = total_courses
    try:
        course1 = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=100)[0]
        context['session'] = course1.session
        context['semester'] = course1.semester
    except:
        pass

    return render(request, 'management/undergrad_courses_registered_100_harm.html', context=context)

@login_required
def showCoursesRegisteredUnderGrad200Harm(request):
    context = {}
    filtered_courses = CourseRegisteredUnderGradFilter(
        request.GET,
        queryset = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=200)
    )
    context['filtered_courses'] = filtered_courses
    paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
    page_number = request.GET.get('page')
    courses_page_obj = paginated_filtered_courses.get_page(page_number)
    context['courses_page_obj'] = courses_page_obj
    total_courses = filtered_courses.qs.count()
    context['total_courses'] = total_courses
    try:
        course1 = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=200)[0]
        context['session'] = course1.session
        context['semester'] = course1.semester
    except:
        pass
    return render(request, 'management/undergrad_courses_registered_200_harm.html', context=context)

@login_required
def showCoursesRegisteredUnderGrad300Harm(request):
    context = {}
    filtered_courses = CourseRegisteredUnderGradFilter(
        request.GET,
        queryset = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=300)
    )
    context['filtered_courses'] = filtered_courses
    paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
    page_number = request.GET.get('page')
    courses_page_obj = paginated_filtered_courses.get_page(page_number)
    context['courses_page_obj'] = courses_page_obj
    total_courses = filtered_courses.qs.count()
    context['total_courses'] = total_courses
    try:
        course1 = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=300)[0]
        context['session'] = course1.session
        context['semester'] = course1.semester
    except:
        pass

    return render(request, 'management/undergrad_courses_registered_300_harm.html', context=context)

@login_required
def showCoursesRegisteredUnderGrad400Harm(request):
    context = {}
    filtered_courses = CourseRegisteredUnderGradFilter(
        request.GET,
        queryset = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=400)
    )
    context['filtered_courses'] = filtered_courses
    paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
    page_number = request.GET.get('page')
    courses_page_obj = paginated_filtered_courses.get_page(page_number)
    context['courses_page_obj'] = courses_page_obj
    total_courses = filtered_courses.qs.count()
    context['total_courses'] = total_courses
    try:
        course1 = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Harmattan", static_level=400)[0]
        context['session'] = course1.session
        context['semester'] = course1.semester
    except:
        pass

    return render(request, 'management/undergrad_courses_registered_400_harm.html', context=context)

@login_required
def showCoursesRegisteredUnderGrad100Rain(request):
    context = {}
    filtered_courses = CourseRegisteredUnderGradFilter(
        request.GET,
        queryset = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Rain", static_level=100)
    )
    context['filtered_courses'] = filtered_courses
    paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
    page_number = request.GET.get('page')
    courses_page_obj = paginated_filtered_courses.get_page(page_number)
    context['courses_page_obj'] = courses_page_obj
    total_courses = filtered_courses.qs.count()
    context['total_courses'] = total_courses
    try:
        course1 = CourseRegistration.objects.filter(undergrad=request.user, semester__semester="Rain", static_level=100)[0]
        context['session'] = course1.session
        context['semester'] = course1.semester
    except:
        pass

    return render(request, 'management/undergrad_courses_registered_100_rain.html', context=context)

def exportCSVCourse(request, id):
    course = Course.objects.get(id=id)
    programmes = course.programme.all()

    students = Person.objects.filter(type="Undergraduate", level=course.level, programme__in=programmes)
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename=Students for ' + str(course.course_code) + '.csv'

    writer = csv.writer(response)
    writer.writerow(['Matric Numbers', "Names of Students", "Programme", 'Course', 'CA', 'Exam'])

    for each in students:
        if each.middle_name == None:
            writer.writerow(
                [each.username, str(each.last_name) + " " + str(each.first_name), each.programme, course.course_code]
            )
        else:
            writer.writerow(
                [each.username, str(each.last_name) + " " + str(each.first_name) + " " + str(each.middle_name), each.programme, course.course_code]
            )

    return response

def decode_utf8(line_iterator):
    for line in line_iterator:
        yield line.decode('utf-8')

def uploadCSV(request, id):
    course = Course.objects.get(id=id)
    if course.lecturer == request.user:
        reg99 = Semester.objects.get(current=True)
        reg98 = Session.objects.get(current=True)

        if course in reg99.course_recorded.all():
            messages.success(request, "Scores have been uploaded for " +str(course.course_code)+ " for the semester")
            return redirect('courses_academics')
        else:
            if request.method == 'GET':
                form = UploadForm()
                return render(request, 'management/upload.html', {'form': form, 'course':course})

            form = UploadForm(request.POST, request.FILES)
            # Validate the form
            if form.is_valid():
                # Get the correct type string instead of byte without reading full file into memory with a generator to decode line by line
                products_file = csv.reader(decode_utf8(request.FILES.get('sent_file')))
                next(products_file)  # Skip header row

                for counter, line in enumerate(products_file):
                    Matric = line[0]
                    programme = line[2]
                    d_course = line[3]
                    if line[4] =="":
                        CA = None
                    else:
                        CA = line[4]
                    if line[5] == "":
                        Exam = None
                    else:
                        Exam = line[5]

                    e_course = Course.objects.get(lecturer=request.user, course_code=d_course)
                    undergrad = Person.objects.get(username=Matric)
                    if course != e_course:
                        messages.error(request, "You're trying to upload " +str(e_course.course_code)+ " sheet into " +str(course.course_code))
                        return redirect('courses_academics')

                    p = Score()
                    p.session = reg98
                    p.semester = reg99
                    p.poster = request.user
                    p.static_lecturer = request.user
                    p.undergrad = Person.objects.get(username=Matric)
                    p.course = course
                    p.ca1 = CA
                    p.exam = Exam

                    try:
                        p.total = int(CA) + int(Exam)
                        p.credit_unit = course.credit_unit
                    except:
                        p.total = None
                        p.credit_unit = 0
                    try:
                        total = int(CA) + int(Exam)
                    except:
                        total = None
                    try:
                        if total >= 70:
                            points = course.credit_unit * 5
                            p.points = points
                        elif total >= 60:
                            points = course.credit_unit * 4
                            p.points = points
                        elif total >= 50:
                            points = course.credit_unit * 3
                            p.points = points
                        elif total >= 45:
                            points = course.credit_unit * 2
                            p.points = points
                        elif total >= 40:
                            points = course.credit_unit * 1
                            p.points = points
                        else:
                            points = course.credit_unit * 0
                            p.points = points
                    except:
                        p.points = 0
                    p.save()
                    reg95 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)
                    reg91 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad)

                    try:
                        reg97 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad).aggregate(Sum('points'))['points__sum']
                        reg96 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad).aggregate(Sum('credit_unit'))['credit_unit__sum']
                        for each in reg95:
                            each.semester_points = reg97
                            each.semester_credits = reg96
                            each.save()
                    except:
                        pass
                    try:
                        reg94 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad).aggregate(Sum('points'))['points__sum']
                        reg93 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad).aggregate(Sum('credit_unit'))['credit_unit__sum']
                        for each in reg91:
                            each.total_points = reg94
                            each.total_credits = reg93
                            each.save()
                    except:
                        pass

                    reg10 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)[0]
                    try:
                        semester_gpa = reg10.semester_points/reg10.semester_credits
                    except:
                        semester_gpa = 0
                    try:
                        cumulative_gpa = reg10.total_points/reg10.total_credits
                    except:
                        cumulative_gpa = 0
                    reg11 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)
                    reg12 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad)

                    for each2 in reg11:
                        each2.semester_gpa = semester_gpa
                        each2.static_level = str(each2.undergrad.level)
                        each2.static_adviser = each2.undergrad.programme.department.adviser
                        each2.static_dean = each2.undergrad.programme.department.faculty.dean
                        each2.static_undergrad_image = each2.undergrad.photograph
                        each2.save()
                    for each2 in reg12:
                        each2.cumulative_gpa = cumulative_gpa
                        each2.save()

                messages.success(request, "Scores have been uploaded for " +str(course.course_code)+ " for the semester")
                reg99.course_recorded.add(course)

                return redirect('upload_csv', id)
            else:
                return redirect('index')

def uploadCSVAdm(request, id):
    course = Course.objects.get(id=id)
    if request.user.type == "A--":
        reg99 = Semester.objects.get(current=True)
        reg98 = Session.objects.get(current=True)

        if request.method == 'GET':
            form = UploadForm()
            return render(request, 'management/upload_adm.html', {'form': form, 'course':course})

        form = UploadForm(request.POST, request.FILES)
        # Validate the form
        if form.is_valid():
            # Get the correct type string instead of byte without reading full file into memory with a generator to decode line by line
            products_file = csv.reader(decode_utf8(request.FILES.get('sent_file')))
            next(products_file)  # Skip header row

            for counter, line in enumerate(products_file):
                Matric = line[0]
                programme = line[2]
                d_course = line[3]
                if line[4] =="":
                    CA = None
                else:
                    CA = line[4]
                if line[5] == "":
                    Exam = None
                else:
                    Exam = line[5]

                e_course = Course.objects.get(course_code=d_course)
                undergrad = Person.objects.get(username=Matric)
                if course != e_course:
                    messages.error(request, "You're trying to upload " +str(e_course.course_code)+ " sheet into " +str(course.course_code))
                    return redirect('courses_academics')

                p = Score()
                p.session = reg98
                p.semester = reg99
                p.poster = request.user
                p.static_lecturer = request.user
                p.undergrad = Person.objects.get(username=Matric)
                p.course = course
                p.ca1 = CA
                p.exam = Exam

                try:
                    p.total = int(CA) + int(Exam)
                    p.credit_unit = course.credit_unit
                except:
                    p.total = None
                    p.credit_unit = 0
                try:
                    total = int(CA) + int(Exam)
                except:
                    total = None
                try:
                    if total >= 70:
                        points = course.credit_unit * 5
                        p.points = points
                    elif total >= 60:
                        points = course.credit_unit * 4
                        p.points = points
                    elif total >= 50:
                        points = course.credit_unit * 3
                        p.points = points
                    elif total >= 45:
                        points = course.credit_unit * 2
                        p.points = points
                    elif total >= 40:
                        points = course.credit_unit * 1
                        p.points = points
                    else:
                        points = course.credit_unit * 0
                        p.points = points
                except:
                    p.points = 0

                p.save()

                reg95 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)
                reg91 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad)
                try:
                    reg97 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad).aggregate(Sum('points'))['points__sum']
                    reg96 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad).aggregate(Sum('credit_unit'))['credit_unit__sum']
                    for each in reg95:
                        each.semester_points = reg97
                        each.semester_credits = reg96
                        each.save()
                except:
                    pass
                try:
                    reg94 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad).aggregate(Sum('points'))['points__sum']
                    reg93 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad).aggregate(Sum('credit_unit'))['credit_unit__sum']
                    for each in reg91:
                        each.total_points = reg94
                        each.total_credits = reg93
                        each.save()
                except:
                    pass

                reg10 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)[0]
                try:
                    semester_gpa = reg10.semester_points/reg10.semester_credits
                except:
                    semester_gpa = 0
                try:
                    cumulative_gpa = reg10.total_points/reg10.total_credits
                except:
                    cumulative_gpa = 0
                reg11 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)
                reg12 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad)

                for each2 in reg11:
                    each2.semester_gpa = semester_gpa
                    each2.static_level = str(each2.undergrad.level)
                    each2.static_adviser = each2.undergrad.programme.department.adviser
                    each2.static_dean = each2.undergrad.programme.department.faculty.dean
                    each2.static_undergrad_image = each2.undergrad.photograph
                    each2.save()
                for each2 in reg12:
                    each2.cumulative_gpa = cumulative_gpa
                    each2.save()

            messages.success(request, "Scores have been uploaded for " +str(course.course_code)+ " for the semester")

            reg99.course_recorded.add(course)

            return redirect('upload_csv_adm', id)
        else:
            return redirect('index')

def uploadSpec(request, id):
    course = Course.objects.get(id=id)
    if request.user.type == "A--":
        reg99 = Semester.objects.get(current=True)
        reg98 = Session.objects.get(current=True)

        if request.method == 'GET':
            form = UploadForm()
            return render(request, 'management/upload.html', {'form': form, 'course':course})

        form = UploadForm(request.POST, request.FILES)
        # Validate the form
        if form.is_valid():
            # Get the correct type string instead of byte without reading full file into memory with a generator to decode line by line
            products_file = csv.reader(decode_utf8(request.FILES.get('sent_file')))
            next(products_file)  # Skip header row

            for counter, line in enumerate(products_file):
                Matric = line[0]
                programme = line[2]
                d_course = line[3]
                if line[4] =="":
                    CA = 0
                else:
                    CA = line[4]
                if line[5] == "":
                    Exam = 0
                else:
                    Exam = line[5]

                e_course = Course.objects.get(course_code=d_course)
                undergrad = Person.objects.get(username=Matric)
                if course != e_course:
                    messages.error(request, "You're trying to upload " +str(e_course.course_code)+ " sheet into " +str(course.course_code))
                    return redirect('courses_academics')

                p = Score()
                p.session = reg98
                p.semester = reg99
                p.poster = request.user
                p.static_lecturer = request.user
                p.undergrad = Person.objects.get(username=Matric)
                p.course = course
                p.ca1 = CA
                p.exam = Exam

                try:
                    p.total = int(CA) + int(Exam)
                except:
                    p.total = 0
                p.credit_unit = course.credit_unit

                try:
                    total = int(CA) + int(Exam)
                except:
                    total = 0

                if total >= 70:
                    points = course.credit_unit * 5
                    p.points = points
                elif total >= 60:
                    points = course.credit_unit * 4
                    p.points = points
                elif total >= 50:
                    points = course.credit_unit * 3
                    p.points = points
                elif total >= 45:
                    points = course.credit_unit * 2
                    p.points = points
                elif total >= 40:
                    points = course.credit_unit * 1
                    p.points = points
                else:
                    points = course.credit_unit * 0
                    p.points = points

                p.save()

                reg95 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)
                reg91 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad)

                try:
                    reg97 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad).aggregate(Sum('points'))['points__sum']
                    reg96 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad).aggregate(Sum('credit_unit'))['credit_unit__sum']
                    for each in reg95:
                        each.semester_points = reg97
                        each.semester_credits = reg96
                        each.save()
                except:
                    pass
                try:
                    reg94 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad).aggregate(Sum('points'))['points__sum']
                    reg93 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad).aggregate(Sum('credit_unit'))['credit_unit__sum']
                    for each in reg91:
                        each.total_points = reg94
                        each.total_credits = reg93
                        each.save()
                except:
                    pass

                reg10 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)[0]
                semester_gpa = reg10.semester_points/reg10.semester_credits

                try:
                    cumulative_gpa = reg10.total_points/reg10.total_credits
                except:
                    cumulative_gpa = 0

                reg11 = Score.objects.filter(~Q(recorded="Y"), session=reg98, semester=reg99, undergrad=undergrad)
                reg12 = Score.objects.filter(~Q(recorded="Y"), undergrad=undergrad)

                for each2 in reg11:
                    each2.semester_gpa = semester_gpa
                    each2.static_level = str(each2.undergrad.level)
                    each2.static_adviser = each2.undergrad.programme.department.adviser
                    each2.static_dean = each2.undergrad.programme.department.faculty.dean
                    each2.static_undergrad_image = each2.undergrad.photograph
                    each2.save()
                for each2 in reg12:
                    each2.cumulative_gpa = cumulative_gpa
                    each2.save()

            messages.success(request, "Scores have been uploaded for " +str(course.course_code)+ " for the semester")

            reg99.course_recorded.add(course)

            return redirect('upload_csv', id)
        else:
            return redirect('index')

@login_required
def registerCourseUnderGrad100Harm(request, id):
    course = Course.objects.get(id=id)
    course_code = course.course_code
    form = CourseFormUnderGrad()
    if request.method == 'POST':
        form = CourseFormUnderGrad(request.POST or None)
        if form.is_valid():
            form.save(commit=False).course = course
            form.save(commit=False).undergrad = request.user
            # form.save(commit=False).semester = "Rain"
            form.save()
            session = Session.objects.get(current=True)
            semester = Semester.objects.get(current=True)
            reg3 = CourseRegistration.objects.filter(undergrad=request.user, course=course)[0]
            # semester = reg4.semester
            reg3.semester = semester
            reg3.session = session
            reg3.save()
            messages.success(request, "The course has been registered successfully")



            # try:
            #     reg2 = CourseRegistration.objects.filter(session=reg5, semester=reg4, undergrad=request.user, course=course)[0]
            #     messages.error(request, "You have already registered for " + str(course_code))
            #     reg2.delete()
            # except:
            #     pass


            # reg = CourseRegistration.objects.filter(undergrad=request.user, course=course)[0]


            return redirect('courses_undergrads_100_harm')
    return render(request, 'management/undergrad_course_form_100_harm.html', {'form': form, 'course': course, 'session': session})

@login_required
def showInvoicesAdmin(request):
    if request.user.type == "A--":
        context = {}
        filtered_invoices = InvoiceAdminFilter(
            request.GET,
            queryset = Invoice.objects.all()
        )
        context['filtered_invoices'] = filtered_invoices
        paginated_filtered_invoices = Paginator(filtered_invoices.qs, 500)
        page_number = request.GET.get('page')
        invoices_page_obj = paginated_filtered_invoices.get_page(page_number)
        context['invoices_page_obj'] = invoices_page_obj
        total_invoices = filtered_invoices.qs.count()
        context['total_invoices'] = total_invoices

        # form = AntiOrderForm()
        #
        # if request.method == 'POST':
        #     form = AntiOrderForm(request.POST, request.FILES, None)
        #     if form.is_valid():
        #         outlet = Outlet.objects.get(partner=request.user)
        #
        #         user = form.cleaned_data.get('user')
        #         cylinder = form.cleaned_data.get('cylinder')
        #
        #         payment1 = form.cleaned_data.get('payment1')
        #         payment2 = form.cleaned_data.get('payment2')
        #         payment3 = form.cleaned_data.get('payment3')
        #
        #         x = datetime.datetime.now().year + datetime.datetime.now().month + datetime.datetime.now().day + datetime.datetime.now().hour + datetime.datetime.now().minute + datetime.datetime.now().second + 50*datetime.datetime.now().microsecond
        #
        #         form.save(commit=False).outlet = outlet
        #         form.save(commit=False).order_Id = x
        #         form.save(commit=False).payment_total = payment1
        #         form.save(commit=False).stage = "1st"
        #
        #         form.save()
        #
        #         reg = AntiOrder.objects.filter(user=user)[0]
        #         total = 0
        #         total_cylinder = ""
        #         static_price2 = ""
        #
        #         for each in reg.cylinder.all():
        #             total += each.category.price
        #             static_price2 += "N" + str(each.category.price) + " "
        #             total_cylinder = total_cylinder + "" + each.product_Id
        #         reg.static_total_cost2 = total
        #         reg.static_price2 = static_price2
        #         reg.save()
        #
        #         owing_entry = Owing()
        #         owing_entry.customer = user
        #         owing_entry.cylinder = total_cylinder
        #         owing_entry.save()
        #
        #         owings = Owing.objects.filter(customer=user)
        #         reg2 = Person.objects.get(username=user.username)
        #         reg2.holding = ""
        #         reg2.save()
        #         reg1 = Person.objects.get(username=user.username)
        #         for each in owings:
        #             reg1.holding = reg1.holding + "" + each.cylinder
        #             reg1.save()
        #
        #         reg3 = AntiOrder.objects.filter(user=user)[0]
        #         reg3.balance = reg3.static_total_cost2 - reg3.payment_total
        #         reg3.payment_date = reg3.updated
        #         reg3.save()
        #
        #
        #         reg4 = AntiOrder.objects.filter(user=user)[0]
        #         reg5 = Product.objects.filter(vendor_product_status="Released Filled to QwikPartner", partner_product_status="Unselected")
        #
        #         for each1 in reg4.cylinder.all():
        #             for each2 in reg5:
        #                 if each2.product_Id == each1.product_Id:
        #                     each2.partner_product_status = "Selected"
        #                     each2.save()
        #
        #         messages.success(request, "The anticipatory order has been added successfully")
        #         return redirect('anticipate:qwikpartner_anti_orders')
        #     else:
        #         messages.error(request, "Please review form input fields below")
        # context['form'] = form


        return render(request, 'management/admin_invoices.html', context=context)
    else:
        return redirect('index')

@login_required
def showAdminInvoice(request, id):
    if request.user.type=="A--":
        invoice = Invoice.objects.get(id=id)
        return render(request, 'management/invoice.html', {'invoice':invoice})
    else:
        return redirect('index')

@login_required
def allInvoices(request):
    if request.user.type=="A--":
        unders = Person.objects.filter(level__level=100)
        # new_entry = Invoice()
        # new_entry.undergrad = request.user
        # new_entry.save()

        for each in unders:
            new_entry = Invoice()
            new_entry.undergrad = each
            new_entry.save()

        return redirect('invoices_admin')
    else:
        return redirect('index')

@login_required
def showAdminSpreadsheets(request):
    if request.user.type == 'A--':
        context = {}
        filtered_programmes = ProgrammeFilter(
            request.GET,
            queryset = Spreadsheet.objects.all().order_by('programme__programme')
        )
        context['filtered_programmes'] = filtered_programmes
        paginated_filtered_programmes = Paginator(filtered_programmes.qs, 50)
        page_number = request.GET.get('page')
        programmes_page_obj = paginated_filtered_programmes.get_page(page_number)
        context['programmes_page_obj'] = programmes_page_obj
        total_programmes = filtered_programmes.qs.count()
        context['total_programmes'] = total_programmes
        return render(request, 'management/admin_spreadsheets.html', context=context)
    else:
        return redirect('index')

@login_required
def showSpreadsheetsHoD(request):
    if request.user.type == 'HoD':
        context = {}
        filtered_programmes = ProgrammeFilter(
            request.GET,
            queryset = Spreadsheet.objects.filter(programme__department__hod=request.user).order_by('-semester__created', 'level__level', 'programme__programme'),
            request=request,
        )
        context['filtered_programmes'] = filtered_programmes
        paginated_filtered_programmes = Paginator(filtered_programmes.qs, 50)
        page_number = request.GET.get('page')
        programmes_page_obj = paginated_filtered_programmes.get_page(page_number)
        context['programmes_page_obj'] = programmes_page_obj
        total_programmes = filtered_programmes.qs.count()
        context['total_programmes'] = total_programmes
        return render(request, 'management/hod_spreadsheets.html', context=context)
    else:
        return redirect('index')

@login_required
def showSpreadsheetsDean(request):
    if request.user.type == 'Dean':
        context = {}
        filtered_programmes = ProgrammeFilterDean(
            request.GET,
            queryset = Spreadsheet.objects.filter(programme__department__faculty__the_dean=request.user).order_by('programme__programme'),
            request=request,
        )
        context['filtered_programmes'] = filtered_programmes
        paginated_filtered_programmes = Paginator(filtered_programmes.qs, 50)
        page_number = request.GET.get('page')
        programmes_page_obj = paginated_filtered_programmes.get_page(page_number)
        context['programmes_page_obj'] = programmes_page_obj
        total_programmes = filtered_programmes.qs.count()
        context['total_programmes'] = total_programmes
        return render(request, 'management/dean_spreadsheets.html', context=context)
    else:
        return redirect('index')

@login_required
def showSpreadsheetsVC(request):
    if request.user.type == 'VC':
        context = {}
        filtered_programmes = ProgrammeFilterVC(
            request.GET,
            queryset = Spreadsheet.objects.all().order_by('programme__programme')
        )
        context['filtered_programmes'] = filtered_programmes
        paginated_filtered_programmes = Paginator(filtered_programmes.qs, 50)
        page_number = request.GET.get('page')
        programmes_page_obj = paginated_filtered_programmes.get_page(page_number)
        context['programmes_page_obj'] = programmes_page_obj
        total_programmes = filtered_programmes.qs.count()
        context['total_programmes'] = total_programmes
        return render(request, 'management/vc_spreadsheets.html', context=context)
    else:
        return redirect('index')

@login_required
def addSpreadsheet(request):
    form = SpreadsheetForm()
    if request.method=='POST':
        form = SpreadsheetForm(request.POST or None)
        if form.is_valid():
            form.save()
            session = form.cleaned_data.get('session')

            semesters = Semester.objects.filter(session=session)
            levels = Level.objects.all()
            programmes = Programme.objects.all()
            for each in semesters:
                for each2 in levels:
                    for each3 in programmes:
                        new_entry = Spreadsheet()
                        new_entry.programme = each3
                        new_entry.level = each2
                        new_entry.semester = each
                        new_entry.session = session
                        new_entry.save()

            reg = Spreadsheet.objects.get(semester=None)
            reg.delete()

        messages.success(request, "Added!")
    else:
        messages.error(request, "Please review form input fields below")

    return render(request, 'management/spreadsheet.html', {'form': form})

@login_required
def updateCoursesHoD(request, id):
    course = Course.objects.get(id=id)
    if request.user.type == "HoD":
        form = CourseFormHoD(instance=course, request=request)
        if request.method=='POST':
            form = CourseFormHoD(request.POST, instance=course, request=request)
            if form.is_valid():
                form.save()
                messages.success(request, "The course has been updated successfully")
                return redirect('update_hod_courses', id)
    else:
        messages.success(request, "You are not authorised")
        return redirect('index')

    return render(request, 'management/update_hod_course.html', {'form': form, 'course':course})

@login_required
def addCourseHoD(request):
    if request.user.type == "HoD":
        form = CourseFormHoD(request=request)
        if request.method == 'POST':
            form = CourseFormHoD(request.POST, request.FILES, None, request=request)
            if form.is_valid():
                form.save(commit=False).department=request.user.department
                form.save()
                messages.success(request, "The course has been added successfully")
                return redirect('hod_course')
            else:
                messages.error(request, "Please review form input fields below")

    else:
        messages.success(request, "You are not authorised")
        return redirect('index')

    return render(request, 'management/hod_course.html', {'form': form})

@login_required
def showCoursesHoD(request):
    if request.user.type == "HoD":
        context = {}
        filtered_courses = CourseFilterHoD(
            request.GET,
            queryset = Course.objects.filter(lecturer__department__hod=request.user).order_by('course_code')
        )
        context['filtered_courses'] = filtered_courses
        paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
        page_number = request.GET.get('page')
        courses_page_obj = paginated_filtered_courses.get_page(page_number)
        context['courses_page_obj'] = courses_page_obj
        total_courses = filtered_courses.qs.count()
        context['total_courses'] = total_courses

        return render(request, 'management/hod_courses.html', context=context)
    else:
        return redirect('index')

@login_required
def showCoursesDean(request):
    if request.user.type == "Dean":
        context = {}
        filtered_courses = CourseFilterDean(
            request.GET,
            queryset = Course.objects.filter(lecturer__department__faculty__the_dean=request.user).order_by('course_code')
        )
        context['filtered_courses'] = filtered_courses
        paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
        page_number = request.GET.get('page')
        courses_page_obj = paginated_filtered_courses.get_page(page_number)
        context['courses_page_obj'] = courses_page_obj
        total_courses = filtered_courses.qs.count()
        context['total_courses'] = total_courses

        return render(request, 'management/dean_courses.html', context=context)
    else:
        return redirect('index')

@login_required
def showCoursesVC(request):
    if request.user.type == "VC":
        context = {}
        filtered_courses = CourseFilterDean(
            request.GET,
            queryset = Course.objects.filter(open=True).order_by('course_code')
        )
        context['filtered_courses'] = filtered_courses
        paginated_filtered_courses = Paginator(filtered_courses.qs, 50)
        page_number = request.GET.get('page')
        courses_page_obj = paginated_filtered_courses.get_page(page_number)
        context['courses_page_obj'] = courses_page_obj
        total_courses = filtered_courses.qs.count()
        context['total_courses'] = total_courses

        return render(request, 'management/vc_courses.html', context=context)
    else:
        return redirect('index')

@login_required
def showSemestersHoD(request):
    if request.user.type == "HoD":
        context = {}
        filtered_semesters = SemesterFilter(
            request.GET,
            queryset = Semester.objects.all()
        )
        context['filtered_semesters'] = filtered_semesters
        paginated_filtered_semesters = Paginator(filtered_semesters.qs, 20)
        page_number = request.GET.get('page')
        semesters_page_obj = paginated_filtered_semesters.get_page(page_number)
        context['semesters_page_obj'] = semesters_page_obj
        total_semesters = filtered_semesters.qs.count()
        context['total_semesters'] = total_semesters
        faculty = request.user.department.faculty
        context['faculty'] = faculty
        return render(request, 'management/hod_semesters.html', context=context)
    else:
        return redirect('index')

@login_required
def showSemestersDean(request):
    if request.user.type == "Dean":
        context = {}
        filtered_semesters = SemesterFilter(
            request.GET,
            queryset = Semester.objects.all()
        )
        context['filtered_semesters'] = filtered_semesters
        paginated_filtered_semesters = Paginator(filtered_semesters.qs, 20)
        page_number = request.GET.get('page')
        semesters_page_obj = paginated_filtered_semesters.get_page(page_number)
        context['semesters_page_obj'] = semesters_page_obj
        total_semesters = filtered_semesters.qs.count()
        context['total_semesters'] = total_semesters
        depts = Department.objects.filter(faculty__the_dean=request.user)
        context['depts'] = depts
        return render(request, 'management/dean_semesters.html', context=context)
    else:
        return redirect('index')

@login_required
def showSemestersVC(request):
    if request.user.type == "VC":
        context = {}
        filtered_semesters = SemesterFilter(
            request.GET,
            queryset = Semester.objects.all()
        )
        context['filtered_semesters'] = filtered_semesters
        paginated_filtered_semesters = Paginator(filtered_semesters.qs, 20)
        page_number = request.GET.get('page')
        semesters_page_obj = paginated_filtered_semesters.get_page(page_number)
        context['semesters_page_obj'] = semesters_page_obj
        total_semesters = filtered_semesters.qs.count()
        context['total_semesters'] = total_semesters
        return render(request, 'management/vc_semesters.html', context=context)
    else:
        return redirect('index')

@login_required
def approveHoD(request, id):
    if request.user.type == "HoD":
        semester = Semester.objects.get(id=id)
        dept = request.user.department
        semester.hod_approval.add(dept)
        messages.success(request, "You have approved successfully")
        return redirect('hod_semesters')
    else:
        return redirect('index')

@login_required
def approveDean(request, id):
    if request.user.type == "Dean":
        semester = Semester.objects.get(id=id)
        faculty = request.user.department.faculty
        semester.dean_approval.add(faculty)
        messages.success(request, "You have approved successfully")
        return redirect('dean_semesters')
    else:
        return redirect('index')

@login_required
def approveVC(request, id):
    if request.user.type == "VC":
        semester = Semester.objects.get(id=id)
        semester.vc_approval = True
        semester.save()
        messages.success(request, "You have approved successfully")
        return redirect('vc_semesters')
    else:
        return redirect('index')
